from time import sleep
from subprocess import Popen, PIPE 

from IPython.core.magic import (Magics, magics_class, line_magic,
                                line_cell_magic)
from IPython.testing.skipdoctest import skip_doctest
from IPython.display import Image, display

import numpy as np
from PIL import Image as PImage



@magics_class
class RGBMagics(Magics):

    def __init__(self, shell):
        super(RGBMagics, self).__init__(shell)
        self.proc = None
        self.again = 2
        self.atime = 192
        self.rgbc_count = 65535
        
    @skip_doctest
    @line_magic
    def rgb(self, line):
        if line == "open":
            self.proc = Popen(
                ["python", "ubit/tcs34725_qt.py"],
                stdin = PIPE,
                stdout = PIPE)
        elif line == "close":
            self.proc.kill()
        elif line == "capture":
            if self.proc is not None:
                capture_cmd = "capture\n" 
                self.proc.stdin.write(capture_cmd.encode('utf-8'))
                self.proc.stdin.flush()
                sleep(0.2)
                rgbc_bytes = self.proc.stdout.readline()
                rgbc = eval(rgbc_bytes.rstrip())
                self.max_rgbc_count(self.atime)
                rgb = np.asarray(rgbc[0:3])/self.rgbc_count*255
                r, g, b =  rgb
                print("Data RGBC: %s" % str(rgbc))
                print("R: %d\nG: %d\nB: %d" % (int(r), int(g), int(b)))
                image_data = np.zeros((256 ,256 ,3), dtype=np.uint8)
                image_data[:,:] = np.asarray(rgb)
                pimage = PImage.fromarray(image_data, 'RGB')
                pimage.save("capture.png")
                display(Image("capture.png"))
        elif line[0:5] == "again":
            if self.proc is not None:
                again_cmd = line + '\n'
                self.proc.stdin.write(again_cmd.encode('utf-8'))
                self.proc.stdin.flush()
        elif line[0:5] == "atime":
            if self.proc is not None:
                atime = int(line.split()[1])
                self.atime = atime
                atime_cmd = line + '\n'
                self.proc.stdin.write(atime_cmd.encode('utf-8'))
                self.proc.stdin.flush()
        elif line == "return":
            return self.proc
        elif line == "poll":
            print(self.proc.poll())
        elif line == "connect":
            self.proc.stdin.write(b"connect\n")
            self.proc.stdin.flush()
            
           
    def max_rgbc_count(self, atime):
        max_count = (256 - atime) * 1024
        if max_count <= 65535:
            self.rgbc_count = max_count
        else:
            self.rgbc_count = 65535                    
            
            
def load_ipython_extension(ip):
    """Load the extension in IPython."""
    ip.register_magics(RGBMagics)

