import time

from IPython.core.magic import (Magics, magics_class, line_magic,
                                line_cell_magic)
from IPython.testing.skipdoctest import skip_doctest

from serial import Serial
from serial.tools.list_ports import comports



class UbConnect(object):

    
    def __init__(self):
        super(UbConnect, self).__init__()

    def find_microbit(self):
        """
        Returns the port for the first micro:bit found connected to the computer.
        """
        MICROBIT_PID = 516
        MICROBIT_VID = 3368
        for port in comports():
            if port.vid == MICROBIT_VID and port.pid == MICROBIT_PID:
                return port.device

    def connect(self):
        """
        Returns a pySerial Serial object to talk to the microbit
        """
        BAUDRATE = 115200
        PARITY = 'N'
        port_device = self.find_microbit()
        if port_device is None:
            print("No micro:bit found")
        else:
            s = Serial(port_device, BAUDRATE, parity=PARITY)
            s.write(b'\x03\x01') # Ctrl-C: interrupt, Ctrl-A: switch to raw REPL
            s.read_until(b'raw REPL')
            s.read_until(b'\r\n>') # Wait for prompt
            return s