import ast
import time

from IPython.core.magic import (Magics, magics_class, line_magic,
                                line_cell_magic)
from IPython.testing.skipdoctest import skip_doctest
from serial import Serial
from serial.tools.list_ports import comports
from ubconnect import UbConnect


@magics_class
class UbMagics(Magics, UbConnect):

    def __init__(self, shell):
        super(UbMagics, self).__init__(shell)
        self.serial = self.connect()
        self.delay = 0.001

    def run_code(self, code):
        '''Run a string of code, return strings for stdout and stderr'''
        self.serial.write(code.encode('utf-8') + b'\x04')
        result = bytearray()
        while not result.endswith(b'\x04>'):
            time.sleep(self.delay)
            result.extend(self.serial.read_all())
        assert result.startswith(b'OK')
        out, err = result[2:-2].split(b'\x04', 1)
        return out.decode('utf-8', 'replace'), err.decode('utf-8', 'replace')
  
    @skip_doctest
    @line_cell_magic
    def ub(self, line, cell=None):
        if cell is None:
            out, err = self.run_code(line)
            if out:
                to_nb = out.rstrip()
                print(to_nb)
            if err:
                to_nb = err.rstrip()
                print(to_nb)
        else:
            out, err = self.run_code(cell)
            if out:
                out_list = out.split('\r\n')[:-1]
                for item in out_list:
                    print(item)
            if err:
                print(err.rstrip())
                
    @skip_doctest
    @line_magic
    def ubp(self, line):
        print_line = 'print(%s)' % line
        out, err = self.run_code(print_line)
        if out:
            to_nb = out.rstrip()
            print(to_nb)
        if err:
            to_nb = err.rstrip()
            print(to_nb)

    @skip_doctest
    @line_magic
    def ubr(self, line):
        print_line = 'print(%s)' % line
        out, err = self.run_code(print_line)
        if out:
            to_nb = out.rstrip()
            try:
                return ast.literal_eval(to_nb)
            except:
                return to_nb
        if err:
            to_nb = err.rstrip()
            print(to_nb)

    @skip_doctest
    @line_magic
    def ubport(self, line):
        if self.serial is not None:
            is_open = self.serial.is_open
            if line == "close":
                if is_open is True:
                    self.serial.close()
                    print("port: closed")
            elif line == "open":
                if is_open is False:
                    self.serial.open()
                    print("port: opened")
            elif line == "isopen":
                print(is_open)
            elif line == "reconnect":
                if is_open is True:
                    self.serial.close()
                self.serial.__del__()
                self.serial = self.connect()
        if self.serial is None:
            if line == "connect":
                self.serial = self.connect()
        if line == "return":
            return self.serial
            
    @skip_doctest
    @line_magic
    def ubrx(self, line):
        n_bytes = self.serial.in_waiting
        result = self.serial.read(n_bytes)
        return result
        
    @skip_doctest
    @line_magic
    def ubtx(self, line):
        code = line
        to_serial = self.serial.write(code.encode('utf-8') + b'\x04')
        
    @skip_doctest
    @line_magic
    def ubser(self, line):
        to_serial = "self.serial.%s" % line
        return eval(to_serial)
    
        

def load_ipython_extension(ip):
    """Load the extension in IPython."""
    ip.register_magics(UbMagics)

