import struct
from microbit import i2c, uart, sleep, button_a


const = lambda x: x
address = const(0x29)


def cmd(register):
    command_bit7 = 0x80
    command_byte = command_bit7 | register
    command_code = struct.pack('<B', command_byte)
    return command_code


def read8(register):
    command_byte = cmd(register)
    i2c.write(address, command_byte)
    data8 = i2c.read(address, 1)
    return data8


def write_data(register, data_byte):
    command_code = cmd(register)
    data_to_write = struct.pack('<B', data_byte)
    bytes_to_write = command_code + data_to_write
    i2c.write(address, bytes_to_write)


def enable(on=True):
    _REGISTER_ENABLE = 0x00
    PON = 0x01
    AEN = 0x02
    if on:
        write_data(_REGISTER_ENABLE, AEN | PON)
    else:
        write_data(_REGISTER_ENABLE, 0x00)
        

def read16(register):
    command_byte = cmd(register)
    i2c.write(address, command_byte)
    data16 = i2c.read(address, 2)
    return data16


def read_rgbc():
    _REGISTER_CDATA = 0x14
    _REGISTER_RDATA = 0x16
    _REGISTER_GDATA = 0x18
    _REGISTER_BDATA = 0x1A
    _RGBC = [_REGISTER_RDATA, _REGISTER_GDATA, 
             _REGISTER_BDATA, _REGISTER_CDATA]
    rgbc = bytes()
    for register in _RGBC:
        rgbc += read16(register)
    return rgbc
    
    
def set_atime(atime):
    _REGISTER_ATIME = 0x01
    if 0 <= atime < 256:
        write_data(_REGISTER_ATIME, atime)
        
        
def set_again(again):
    _REGISTER_CONTROL = 0x0F
    if 0 <= again < 4:
        write_data(_REGISTER_CONTROL, again)
        

def main():
    set_again(2)
    set_atime(192)
    enable()
    button_a.was_pressed()
    rgbc_to_qt = False

    while True:
        uart_in = uart.any()
        if uart_in:
            cmd = uart.readall()
            try:
                eval(cmd)
            except:
                pass
        if button_a.was_pressed():
            rgbc_to_qt = not rgbc_to_qt
        if rgbc_to_qt:
            rgbc = read_rgbc()
            data_to_send = rgbc + b'\r\n'
            uart.write(data_to_send)
        sleep(50)


main()    